import std.stdio;
import std.stream;
import std.string;
import std.math;
import std.parallelism ;

import gmp;
import gmpp;
import mpfr;

import Ccanvas;


class Mandelbrot
{
  mpf  ox    ;
  mpf  oy    ;
  mpf  zoom  ;
  mpfr_rnd_t rndmd ;

  Ccanvas canvas  ;
  int  cpunum ;

  this(){
    canvas = new Ccanvas ;
    canvas.width =640 ;
    canvas.height=640 ;
    ox = new mpf("0.0") ;
    oy = new mpf("0.0") ;
    zoom = new mpf("1.0") ;
    //writef("tls support = %d\n",mpfr_buildopt_tls_p()) ;
    rndmd = mpfr_get_default_rounding_mode() ;
    //defaultPoolThreads(0) ;
    cpunum=totalCPUs ;
  }

  int getCPUcount(){
    return(totalCPUs) ;
  }

  int setCPUcount(int cpus){
    cpunum=cpus ;
    return(0) ;
  }

  int setCanvasSize(int width, int height){
    if( width<1 || height<1 ) return(-1) ;
    canvas.width =width  ;
    canvas.height=height ;
    canvas.alloc() ;
    return(0) ;
  }

  int setPosition(double ox, double oy){
    this.ox = new mpf(ox) ;
    this.oy = new mpf(oy) ;
    return(0) ;
  }

  int setPosition(mpf ox, mpf oy){
    this.ox = ox ;
    this.oy = oy ;
    return(0) ;
  }

  int setZoom(double zoom){
    this.zoom = new mpf(zoom) ;
    return(0) ;
  }

  int setZoom(mpf zoom){
    this.zoom = zoom ;
    return(0) ;
  }

  int setMaxCount(int maxcount){
    if( canvas !is null ){
      canvas.maxcount=maxcount ;
    }
    return(0) ;
  }

  int getCanvasWidth(){
    return(canvas.width) ;
  }
  
  int getCanvasHeight(){
    return(canvas.height) ;
  }

  int calc(){
    canvas.alloc() ;

    real rox = std.conv.to!(real)(ox.toString()) ;
    real roy = std.conv.to!(real)(oy.toString()) ;
    real rzm = std.conv.to!(real)(zoom.toString()) ;

    auto poolInstance = new TaskPool(cpunum-1) ;
    foreach(y, ref elem; poolInstance.parallel(canvas.array)) {
      //for( int y=0 ; y<canvas.height ; y++ ){
      calc_line(canvas, cast(int)(y), rox, roy, rzm) ;
    }

    poolInstance.stop() ;
    return(0) ;
  }
  
  void calc_line(Ccanvas canvas, int y, real rox, real roy, real rzm){
    for( int x=0 ; x<canvas.width ; x++ ){
      creal C  ;
      creal Z0 ;
      C  = (4.0*(cast(real)(x)-cast(real)(canvas.width )/2.0)/(rzm*canvas.width)+rox) +
	   (4.0*(cast(real)(y)-cast(real)(canvas.height)/2.0)/(rzm*canvas.width)+roy)*1.0i ;
      Z0 = 0.0 + 0.0i ;

      int count=0 ;
      for( int n=0 ; n<=canvas.maxcount ; n++ ){
	creal Z1 = Z0 * Z0 + C ;
        if( std.math.hypot(Z1.im, Z1.re)>1000000.0 ) break ;
	Z0 = Z1 ;
	count++ ;
      }
      if( count<canvas.maxcount ){
	canvas.array[y][x].count = count ;
      }else{
	canvas.array[y][x].count = -1 ;
      }
    }
  }
  
  int calcMPFR(){
    canvas.alloc() ;

    auto poolInstance = new TaskPool(cpunum-1) ;
    foreach(y, ref elem; poolInstance.parallel(canvas.array)) {
    //for( int y=0 ; y<canvas.height ; y++ ){
      calcMPFR_line(canvas, cast(int)(y), mpf_precision, ox.v, oy.v, zoom.v) ;
    }

    poolInstance.stop() ;
    return(0) ;
  }

  void calcMPFR_line(Ccanvas canvas, int y, uint prec, mpf_t mpfox, mpf_t mpfoy, mpf_t mpfzm){
    static mpfr_t gx, gy, gw, gh   ;
    static mpfr_t  c_re,  c_im     ;
    static mpfr_t z0_re, z0_im     ;
    static mpfr_t z1_re, z1_im     ;
    static mpfr_t th               ;
    static mpfr_t tmp1, tmp2, tmp3 ;
    static mpfr_t mpfrox, mpfroy, mpfrzm ;

    mpfr_inits2(prec,
		&gx, &gy, &gw, &gh,
		&c_re, &c_im, &z0_re, &z0_im, &z1_re, &z1_im, &th,
		&tmp1, &tmp2, &tmp3,
		&mpfrox, &mpfroy, &mpfrzm, null) ;
      
    mpfr_set_si( &gw, canvas.width , rndmd ) ;
    mpfr_set_si( &gh, canvas.height, rndmd ) ;
    mpfr_set_f( &mpfrox, &mpfox, rndmd ) ;
    mpfr_set_f( &mpfroy, &mpfoy, rndmd ) ;
    mpfr_set_f( &mpfrzm, &mpfzm, rndmd ) ;
    
    for( int x=0 ; x<canvas.width ; x++ ){
      mpfr_set_si( &gx, x, rndmd ) ; //scope gx = new mpf(x) ;
      mpfr_set_si( &gy, y, rndmd ) ; //scope gy = new mpf(y) ;
      
      //scope mpf c_re ; c_re=(4*(gx-gw/2))/(gz*gw)+ox ;
      mpfr_div_ui( &tmp1, &gw, 2    , rndmd ) ; // gw/2
      mpfr_sub(    &tmp2, &gx, &tmp1, rndmd ) ; // (gx - gw/2)
      mpfr_mul_ui( &tmp2, &tmp2, 4  , rndmd ) ; // (4*(gx - gw/2))
      mpfr_mul( &tmp3, &mpfrzm, &gw , rndmd ) ; // (gz*gw)
      mpfr_div( &c_re, &tmp2, &tmp3 , rndmd ) ; // (4*(gx-gw/2))/(gz*gw)
      mpfr_add( &c_re, &c_re, &mpfrox,rndmd ) ; // (4*(gx-gw/2))/(gz*gw)+ox
      //scope mpf c_im ; c_im=(4*(gy-gh/2))/(gz*gw)+oy ;
      mpfr_div_ui( &tmp1, &gh, 2    , rndmd ) ; // gh/2
      mpfr_sub(    &tmp2, &gy, &tmp1, rndmd ) ; // (gy - gh/2)
      mpfr_mul_ui( &tmp2, &tmp2, 4  , rndmd ) ; // (4*(gy - gh/2))
      mpfr_div( &c_im, &tmp2, &tmp3 , rndmd ) ; // (4*(gy-gh/2))/(gz*gw)
      mpfr_add( &c_im, &c_im, &mpfroy,rndmd ) ; // (4*(gy-gh/2))/(gz*gw)+oy
      //scope  C  = new mpfComplex(c_re,c_im) ;
      
      //scope  Z0 = new mpfComplex(0, 0) ;
      mpfr_set_si( &z0_re, 0, rndmd ) ;
      mpfr_set_si( &z0_im, 0, rndmd ) ;
      
      mpfr_set_d( &th, 1000000.0, rndmd ) ; //scope  TH = new mpf(1000000.0) ;
      
      int count=0 ;
      for( int n=0 ; n<canvas.maxcount ; n++ ){
	//mpfComplex Z1 = Z0 * Z0 + C ;
	mpfr_mul( &z1_re , &z0_re, &z0_re, rndmd ) ;
	mpfr_mul( &tmp1  , &z0_im, &z0_im, rndmd ) ;
	mpfr_sub( &z1_re , &z1_re, &tmp1 , rndmd ) ;
	mpfr_add( &z1_re , &z1_re, &c_re , rndmd ) ;
	mpfr_mul( &z1_im , &z0_re, &z0_im, rndmd ) ;
	mpfr_mul( &tmp1  , &z0_re, &z0_im, rndmd ) ;
	mpfr_add( &z1_im , &z1_im, &tmp1 , rndmd ) ;
	mpfr_add( &z1_im , &z1_im, &c_im , rndmd ) ;
	
	mpfr_mul( &tmp1, &z1_re, &z1_re, rndmd ) ;
	mpfr_mul( &tmp2, &z1_im, &z1_im, rndmd ) ;
	mpfr_add( &tmp1, &tmp1, &tmp2  , rndmd ) ;
	mpfr_sqrt( &tmp1, &tmp1 , rndmd ) ;
	int z1abs_gt_th = mpfr_cmp( &tmp1, &th ) ;
	if( z1abs_gt_th>0 ) break ;
	mpfr_set(&z0_re, &z1_re, rndmd) ;
	mpfr_set(&z0_im, &z1_im, rndmd) ;
	count++ ;
      }
      if( count<canvas.maxcount ){
	canvas.array[y][x].count = count ;
      }else{
	canvas.array[y][x].count = -1 ;
      }
    }
    
    mpfr_clears(&gx, &gy, &gw, &gh,
		&c_re, &c_im, &z0_re, &z0_im, &z1_re, &z1_im, &th,
		&tmp1, &tmp2, &tmp3,
		&mpfrox, &mpfroy, &mpfrzm, null) ;

  }
  
  Ccanvas getCanvas(){
    return(canvas) ;
  }

}
