import win32.windows;
import win32.windef;
import win32frame.WindowObject;
import win32frame.Split;
import win32frame.CheckBox;
import win32frame.PushButton;
import win32frame.MsgBox;

import std.stdio;
import std.string;
import std.utf;

import Spin;
import SubPanel2 ;
import FontMan;

import Mandelbrot ;
import Coloring ;

enum CMD{
  SPINZOOM   = 0x102,
  SPINOX     = 0x103,
  SPINOY     = 0x104,
  SPINMAXCNT = 0x105,
  SPINCSTEP  = 0x106,
  SPINCPHASE = 0x107,
  USEMPFR    = 0x108,
  SPINCPUS   = 0x109,
  RECALC     = 0x10a
}
  
class SubPanel1 : WindowObject {
  HFONT      hFont;
  Spin       spinzoom  ;
  Spin       spinox    ;
  Spin       spinoy    ;
  Spin       maxcount  ;
  Spin       colstep   ;
  Spin       colphase  ;
  Spin       cpus      ;
  CheckBox   useMPFR   ;
  PushButton recalcbtn ;
  SubPanel2  subpanel2 ;
  HFONT      hfont_f10 ;
  HFONT      hfont_f12 ;
  HFONT      hfont_f14 ;
  HFONT      hfont_f16 ;
  
  Mandelbrot mandelbrot ;
  Coloring   mandelcolor ;
  
  this(string name, string ClassName) {
    winc.style          = 0; //CS_HREDRAW | CS_VREDRAW;
    winc.cbClsExtra     = 0;
    winc.cbWndExtra     = 0;
    winc.hCursor        = LoadCursor(null , cast(wchar*)(IDC_ARROW));
    winc.hbrBackground  = CreateSolidBrush(GetSysColor(COLOR_BTNFACE)) ;
    winc.lpszMenuName   = null;
    winc.lpszClassName  = toUTFz!(wchar*)(ClassName) ;
    super(name) ;
    this.registwinc() ;
  }

  LRESULT proc(HWND hwnd , UINT msg , WPARAM wp , LPARAM lp){
    switch (msg) {
    case WM_DESTROY:
      DeleteObject(hfont_f10 );
      DeleteObject(hfont_f12);
      DeleteObject(hfont_f14);
      DeleteObject(hfont_f16);
      break ;
      
    case WM_CREATE:
      FontMan font_f10 ; font_f10.init(10,FIXED_PITCH) ;
      hfont_f10 = font_f10.createHFONT() ;
      FontMan font_f12 ; font_f12.init(12,FIXED_PITCH) ;
      hfont_f12 = font_f12.createHFONT() ;
      FontMan font_f14 ; font_f14.init(14,FIXED_PITCH) ;
      hfont_f14 = font_f14.createHFONT() ;
      FontMan font_f16 ; font_f16.init(16,FIXED_PITCH) ;
      hfont_f16 = font_f16.createHFONT() ;
      
      spinzoom = new Spin("SpinZoom", "SpinZoom", true) ;
      spinzoom.setStyle( WS_CHILD |WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE ) ;
      spinzoom.setPosition(0,20,300,20) ;
      spinzoom.setInfo( mandelbrot.zoom.toString(), "0.1", "1.e200", "1.2", SPINMODE.BY ) ;
      spinzoom.setLabel("Zoom:",44) ;
      addObject(spinzoom, cast(HMENU)(CMD.SPINZOOM), true, false) ;
      spinzoom.setTextFont(hfont_f10) ;
      spinzoom.setLabelFont(hfont_f14) ;

      spinox = new Spin("SpinOX", "SpinOX", true) ;
      spinox.setStyle( WS_CHILD |WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE ) ;
      spinox.setPosition(0,40,300,20) ;
      spinox.setInfo( mandelbrot.ox.toString(), "-5.0", "5.0", (1/(spinzoom.getMpfValue()*10)).toString() ) ;
      spinox.setLabel("OX:",44) ;
      addObject(spinox, cast(HMENU)(CMD.SPINOX), true, false) ;
      spinox.setTextFont(hfont_f10) ;
      spinox.setLabelFont(hfont_f14) ;

      spinoy = new Spin("SpinOY", "SpinOY", true) ;
      spinoy.setStyle( WS_CHILD |WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE ) ;
      spinoy.setPosition(0,60,300,20) ;
      spinoy.setInfo( mandelbrot.oy.toString(), "-5.0", "5.0", (1/(spinzoom.getMpfValue()*10)).toString() ) ;
      spinoy.setLabel("OY:",44) ;
      addObject(spinoy, cast(HMENU)(CMD.SPINOY), true, false) ;
      spinoy.setTextFont(hfont_f10) ;
      spinoy.setLabelFont(hfont_f14) ;

      maxcount = new Spin("SpinMaxCnt", "SpinMaxCnt") ;
      maxcount.setStyle( WS_CHILD |WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE ) ;
      maxcount.setPosition(180,80,140,20) ;
      maxcount.setTextFormat("%.0f") ;
      maxcount.setInfo( mandelbrot.canvas.maxcount, 100, 100000, 100 ) ;
      maxcount.setLabel("MAXcount:",80) ;
      addObject(maxcount, cast(HMENU)(CMD.SPINMAXCNT), false, false) ;
      maxcount.setTextFont(hfont_f12) ;
      maxcount.setLabelFont(hfont_f14) ;

      colstep = new Spin("SpinColStep", "SpinColStep") ;
      colstep.setStyle( WS_CHILD |WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE ) ;
      colstep.setPosition(340,80,140,20) ;
      colstep.setTextFormat("%.0f") ;
      colstep.setInfo( mandelcolor.getStep(), 1, 1500, 1 ) ;
      colstep.setLabel("ColorStep:",80) ;
      addObject(colstep, cast(HMENU)(CMD.SPINCSTEP), false, false) ;
      colstep.setTextFont(hfont_f12) ;
      colstep.setLabelFont(hfont_f14) ;

      colphase = new Spin("SpinColPhase", "SpinColPhase") ;
      colphase.setStyle( WS_CHILD |WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE ) ;
      colphase.setPosition(485,80,140,20) ;
      colphase.setTextFormat("%.0f") ;
      colphase.setInfo( mandelcolor.getPhase(), 0, 1535, 1 ) ;
      colphase.setLabel("ColorPhase:",88) ;
      addObject(colphase, cast(HMENU)(CMD.SPINCPHASE), false, false) ;
      colphase.setTextFont(hfont_f12) ;
      colphase.setLabelFont(hfont_f14) ;

      cpus = new Spin("SpinCPUs", "SpinCPUs") ;
      cpus.setStyle( WS_CHILD |WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE ) ;
      cpus.setPosition(80,80,100,20) ;
      cpus.setTextFormat("%.0f") ;
      cpus.setInfo( mandelbrot.getCPUcount(), 1, mandelbrot.getCPUcount(), 1 ) ;
      cpus.setLabel("CPUs:",40) ;
      addObject(cpus, cast(HMENU)(CMD.SPINCPUS), false, false) ;
      cpus.setTextFont(hfont_f12) ;
      cpus.setLabelFont(hfont_f14) ;

      useMPFR = new CheckBox("Use MPFR") ;
      useMPFR.setPosition(0,80,70,20) ;
      useMPFR.addStyle(BS_AUTOCHECKBOX) ;
      addObject(useMPFR, cast(HMENU)(CMD.USEMPFR), false, false) ;
      useMPFR.setFont(hfont_f12) ;
      useMPFR.setState(0) ;
      
      recalcbtn  = new PushButton("ReCalc") ;
      recalcbtn.setPosition(0,100,100,40) ;
      addObject(recalcbtn, cast(HMENU)(CMD.RECALC),true,false) ;
      recalcbtn.setFont(hfont_f16) ;      

      break ;
      
    case WM_PAINT:
      PAINTSTRUCT ps;
      HDC hdc;
      hdc = BeginPaint(hwnd, &ps);
      SetBkMode(hdc,TRANSPARENT) ;
      SelectObject(hdc , hfont_f12);
      SelectObject(hdc , GetStockObject(SYSTEM_FONT));
      EndPaint(hwnd, &ps);
      break ;
      
    case WM_MOVE:
      break ;
      
    case WM_SIZE:
      break;
      
    case WM_LBUTTONDOWN:
    case WM_LBUTTONUP:
    case WM_LBUTTONDBLCLK:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_RBUTTONDBLCLK:
    case WM_MOUSEMOVE:
      break ;

    case WM_COMMAND:
      switch( LOWORD(wp) ){

      case CMD.SPINZOOM :
	spinox.setStep( (1/(spinzoom.getMpfValue()*10)).toString() ) ;
	spinoy.setStep( (1/(spinzoom.getMpfValue()*10)).toString() ) ;
      case CMD.SPINOX :
      case CMD.SPINOY :
	break ;
      case CMD.USEMPFR :
	break ;
      case CMD.RECALC :
	long starttime = timeGetTime() ;
	if( useMPFR.getState()!=BST_CHECKED ){
	  mandelbrot.setZoom(spinzoom.getValue()) ;
	  mandelbrot.setPosition(spinox.getValue(),spinoy.getValue()) ;
	  mandelbrot.setMaxCount(cast(int)(maxcount.getValue())) ;
	  subpanel2.graphic.setMandelPos(mandelbrot.ox, mandelbrot.oy, mandelbrot.zoom) ;
	  mandelbrot.calc() ;
	}else{
	  mandelbrot.setZoom(spinzoom.getMpfValue()) ;
	  mandelbrot.setPosition(spinox.getMpfValue(),spinoy.getMpfValue()) ;
	  subpanel2.graphic.setMandelPos(mandelbrot.ox, mandelbrot.oy, mandelbrot.zoom) ;
	  mandelbrot.setMaxCount(cast(int)(maxcount.getValue())) ;
	  mandelbrot.calcMPFR() ;
	}
	long endtime = timeGetTime() ;
	long difftime ;
	if( starttime <= endtime ){
	  difftime = endtime - starttime ;
	}else{
	  difftime = starttime + (0x100000000 - endtime) ;
	}
	auto canvas = mandelbrot.getCanvas() ;
	mandelcolor.setStep(cast(int)(colstep.getValue())) ;
	mandelcolor.setPhase(cast(int)(colphase.getValue())) ;
	mandelcolor.coloring(canvas) ;
	uint *dstimg  = subpanel2.graphic.getImageBuffer() ;
	uint[] srcimg = mandelcolor.getImage() ;
	dstimg[0 .. srcimg.length] = srcimg[0..$] ;
	subpanel2.graphic.reDisplay() ;
	auto msgbox = new MsgBox(null, format("ElapsTime : %d.%03d sec",difftime/1000,difftime%1000), "Elaps Time", MB_OK ) ;
	msgbox.create() ;
	break ;
      case CMD.SPINCSTEP :
      case CMD.SPINCPHASE :
	auto canvas = mandelbrot.getCanvas() ;
	mandelcolor.setStep(cast(int)(colstep.getValue())) ;
	mandelcolor.setPhase(cast(int)(colphase.getValue())) ;
	mandelcolor.coloring(canvas) ;
	uint *dstimg  = subpanel2.graphic.getImageBuffer() ;
	uint[] srcimg = mandelcolor.getImage() ;
	dstimg[0 .. srcimg.length] = srcimg[0..$] ;
	subpanel2.graphic.reDisplay() ;
	break ;
      case CMD.SPINCPUS :
	mandelbrot.setCPUcount(cast(int)(cpus.getValue())) ;
	break ;
      case 0x201 : // msg from GrapicFrame
	spinox.setValue(subpanel2.graphic.offx) ;
	spinoy.setValue(subpanel2.graphic.offy) ;
	break ;
	
      default :	break ;
      }
      break;
      
    default :
      return( DefWindowProc(hwnd , msg , wp , lp) );
    }
    return(0);
  }

  int setSubPanel2(SubPanel2 sp2){
    subpanel2 = sp2 ;
    return(0) ;
  }

  int setMandelBase(Mandelbrot mandelbrot, Coloring mandelcolor){
    this.mandelbrot  = mandelbrot ;
    this.mandelcolor = mandelcolor ;
    return(0) ;
  }
  

}
