import win32.windows;
import win32.windef;
import win32.commctrl;
import win32frame.WindowMenu;
import win32frame.StatusBar;
import win32frame.MainWindow ;
import win32frame.MsgBox;
import win32frame.FileDialog;

import std.stdio;
import std.string;
import std.utf;

import FontMan;
import MainPanel ;
import NewCanvasDialog ;

enum FILE{
  ROOT  =0x100,
  NEW   =0x101,
  SAVE  =0x102,
  EXIT  =0x103
} ;
enum HELP{
  ROOT    =0x400,
  VERSION =0x401
} ;

class myMainWindow : MainWindow {
  WindowMenu  menuroot ;
  StatusBar   statusbar ;
  MainPanel   mainpanel ;
  HFONT       hfont_v14;

  this(string name, string ClassName) {
    winc.style          = 0; //CS_HREDRAW | CS_VREDRAW;
    winc.cbClsExtra     = 0;
    winc.cbWndExtra     = 0;
    winc.hIcon          = LoadIcon(null , cast(wchar*)(IDI_APPLICATION));
    winc.hCursor        = LoadCursor(null , cast(wchar*)(IDC_ARROW));
    winc.hbrBackground  = CreateSolidBrush(GetSysColor(COLOR_BTNFACE)) ;
    winc.lpszMenuName   = null;
    winc.lpszClassName  = toUTFz!(wchar*)(ClassName) ;
    super(name,ClassName) ;
  }

  LRESULT proc(HWND hwnd , UINT msg , WPARAM wp , LPARAM lp){
    switch (msg) {
    case WM_DESTROY:
      DeleteObject(hfont_v14);
      PostQuitMessage(0);
      break ;
      
    case WM_CREATE:
      menuroot = new WindowMenu() ;
      menuroot.create() ;
      menuroot.addMenu(             MFT_STRING, "File"     , FILE.ROOT ) ;
      menuroot.addItem( "File",     MFT_STRING, "New"      , FILE.NEW  ) ;
      menuroot.addItem( "File",     MFT_STRING, "Save"     , FILE.SAVE ) ;
      menuroot.addItem( "File",     MFT_STRING, "Exit"     , FILE.EXIT ) ;
      menuroot.addMenu(             MFT_STRING, "Help"     , HELP.ROOT ) ;
      menuroot.addItem( "Help",     MFT_STRING, "Version"  , HELP.VERSION) ;
      
      setMenu(menuroot) ;
      
      statusbar = new StatusBar() ;
      statusbar.addStyle(CCS_BOTTOM | SBARS_SIZEGRIP) ;
      addStatusBar(statusbar, "") ;
      
      FontMan font_v14 ; font_v14.init(14,VARIABLE_PITCH) ;
      
      hfont_v14 = font_v14.createHFONT() ;
      
      mainpanel = new MainPanel("MainPanel","MainPanel") ;
      mainpanel.setPosition(0,0,200,200) ;
      mainpanel.setStyle( WS_CHILD |WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE ) ;
      addObject( mainpanel, null, true, true ) ;

      break ;
      
    case WM_MOVE:
      break ;
      
    case WM_SIZE:
      break;
      
    case WM_LBUTTONDOWN:
    case WM_LBUTTONUP:
    case WM_LBUTTONDBLCLK:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_RBUTTONDBLCLK:
    case WM_MOUSEMOVE:
      break ;

    case WM_COMMAND:
      switch( LOWORD(wp) ){
      case FILE.NEW :
	auto newcanvasdlog = new NewCanvasDialog("NewCanvas", "NewCanvas") ;
        newcanvasdlog.create( WS_DLGFRAME | WS_SYSMENU | WS_VISIBLE,
			      winX+100, winY+100, 204, 150, hwnd, null ) ;
        if( newcanvasdlog.getDlogResult()!=0 ){
	  mainpanel.setCanvasSize(newcanvasdlog.getWidth(),newcanvasdlog.getHeight()) ;
	}
	break ;
      case FILE.SAVE :
	FileDialog fdlog = new FileDialog() ;
	string     fname = fdlog.getSaveFileName("PortableNetworkGraphics {*.png}\0*.png\0",hwnd, OFN_OVERWRITEPROMPT) ;
        if( fname.length!=0 ){
	  mainpanel.saveImage(fname) ;
	}
	break ;
      
      case FILE.EXIT :
	int    msgbox_ret ;
	auto msgbox = new MsgBox(hwnd, "終了しますか？", "確認", MB_YESNO ) ;
	msgbox_ret = msgbox.create() ;
	if( msgbox_ret==IDYES ){
	  SendMessage(hwnd, WM_CLOSE, 0,0) ;
	}
	break ;
	
      case HELP.VERSION :
        auto msgbox = new MsgBox(hwnd, "Mandelbrot View   Ver0.02", "Version", MB_OK ) ;
        msgbox.create() ;
        break ;
	
      default :
	break ;
      }
      break;
      
    default :
      return( DefWindowProc(hwnd , msg , wp , lp) );
    }
    return(0);
  }
}
