/**********************************************************************
  JPEGロード
  $Id:$
**********************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <jpeglib.h>

struct JPG_HANDLE{
  struct jpeg_decompress_struct *cinfo ;
  struct jpeg_error_mgr         *jerr  ;
  FILE            *fp    ;
  unsigned char   *lnbuf ;
} ;


int readinfo_jpg( char *file, int *width, int *height, int *bpp )
{
  FILE	 *fp;  /* File pointer */
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr         jerr ;

  *width =0 ;
  *height=0 ;
  *bpp   =0 ;

  if( (fp=fopen(file, "rb"))==NULL ){
    return(-1) ;
  }
  
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_decompress(&cinfo);
  jpeg_stdio_src(&cinfo, fp);
  jpeg_read_header(&cinfo, TRUE);
  jpeg_start_decompress(&cinfo);

  *width =cinfo.output_width  ;
  *height=cinfo.output_height ;
  *bpp   =cinfo.output_components * 8 ;

  //  jpeg_finish_decompress(&cinfo);
  jpeg_destroy_decompress(&cinfo);
  fclose(fp);

  return(0) ;
}

struct JPG_HANDLE *load_jpg_preproc( char *file )
{

  struct JPG_HANDLE *jpg_handle ;

  if( (jpg_handle = malloc(sizeof(struct JPG_HANDLE)))==NULL ){
    return(NULL) ;
  }

  if( (jpg_handle->fp=fopen(file, "rb"))==NULL ){
    free(jpg_handle) ;
    return(NULL) ;
  }
  
  jpg_handle->cinfo = malloc( sizeof(struct jpeg_decompress_struct) ) ;
  jpg_handle->jerr  = malloc( sizeof(struct jpeg_error_mgr)         ) ;
  if( jpg_handle->cinfo==NULL || jpg_handle->jerr==NULL ){
    free( jpg_handle->jerr  ) ;
    free( jpg_handle->cinfo ) ;
    free( jpg_handle        ) ;
    return(NULL) ;
  }
  
  jpg_handle->cinfo->err = jpeg_std_error(jpg_handle->jerr);
  jpeg_create_decompress(jpg_handle->cinfo);
  jpeg_stdio_src(jpg_handle->cinfo, jpg_handle->fp);
  jpeg_read_header(jpg_handle->cinfo, TRUE);
  jpeg_start_decompress(jpg_handle->cinfo);


  jpg_handle->lnbuf = malloc( jpg_handle->cinfo->output_width  *
			      jpg_handle->cinfo->output_components ) ;
  
  if( jpg_handle->lnbuf==NULL ){
    jpeg_destroy_decompress(jpg_handle->cinfo);
    fclose(jpg_handle->fp   ) ;
    free(  jpg_handle->jerr ) ;
    free(  jpg_handle->cinfo) ;
    free(  jpg_handle       ) ;
    return(NULL) ;
  }
  
  return(jpg_handle) ;
}


int load_jpg_getline( unsigned int  *img, struct JPG_HANDLE *jpg_handle )
{
  int i ;

  if( jpg_handle->cinfo->output_scanline < jpg_handle->cinfo->output_height ){
    jpeg_read_scanlines(jpg_handle->cinfo, (JSAMPARRAY)(&jpg_handle->lnbuf), 1 );
    for( i=0 ; i<jpg_handle->cinfo->output_width ; i++ ){
      img[i] = 0xff000000 | 
	(unsigned int)(jpg_handle->lnbuf[i*jpg_handle->cinfo->output_components + 2])<<16 |
	(unsigned int)(jpg_handle->lnbuf[i*jpg_handle->cinfo->output_components + 1])<<8  |
	(unsigned int)(jpg_handle->lnbuf[i*jpg_handle->cinfo->output_components + 0]) ;
    }
  }else{
    for( i=0 ; i<jpg_handle->cinfo->output_width ; i++ ){
      img[i] = 0x00000000 ;
    }
  }
    
  return(0) ;
}


int load_jpg_postproc( struct JPG_HANDLE *jpg_handle )
{
  //jpeg_finish_decompress( jpg_handle->cinfo);
  jpeg_destroy_decompress(jpg_handle->cinfo);
  fclose(jpg_handle->fp   ) ;
  free(  jpg_handle->lnbuf) ;
  free(  jpg_handle->jerr ) ;
  free(  jpg_handle->cinfo) ;
  free(  jpg_handle       ) ;
  jpg_handle=NULL ;

  return(0) ;
}
  

#ifdef DEBUG
int main(int argc , char *argv[])
{
  char *image ;
  int  width,height,bpp ;
  FILE *fp ;
  
  if( argc!=2 ){
    printf("usage : %s file\n",argv[0]) ;
    exit(0) ;
  }

  printf("LIBJPG ver=%d\n",JPEG_LIB_VERSION) ;

  printf("-----------test----------\n") ;
  if( readinfo_jpg(argv[1], &width, &height, &bpp)==0 ){
    printf("%s w=%d, h=%d, bpp=%d\n",argv[1], width, height, bpp) ;
  }
  struct JPG_HANDLE *jpg_handle ;
  jpg_handle = load_jpg_preproc(argv[1]) ;
  if( jpg_handle==NULL ){
    printf("jpeg file open error\n") ;
    exit(0) ;
  }
  int i,j ;
  unsigned int  *iimg ;
  unsigned char *cimg ;
  iimg=malloc(sizeof(unsigned int)*width) ;
  cimg=malloc(width*(bpp/8)) ;
  fp=fopen("test2.ppm","wb") ;
  fputs( "P6" , fp ) ; fputc( 0x0a , fp ) ;
  fprintf( fp , "%d %d", width, height) ;
  fputc( 0x0a , fp ) ;
  fputs( "255" , fp ) ; fputc( 0x0a , fp ) ;

  for( i=0 ; i<height ; i++ ){
    load_jpg_getline( iimg, jpg_handle ) ;
    for( j=0 ; j<width ; j++ ){
      cimg[j*3+0] = (iimg[j] & 0x000000ff)     ;
      cimg[j*3+1] = (iimg[j] & 0x0000ff00)>>8  ;
      cimg[j*3+2] = (iimg[j] & 0x00ff0000)>>16 ;
    }
    fwrite( cimg , width*(bpp/8) , 1 , fp ) ;
  }
  fclose(fp) ;
  load_jpg_postproc( jpg_handle ) ;
  
  exit(0) ;
}
#endif
